/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
import java.io.*;

import javax.microedition.io.Connector;
import javax.microedition.jcrmi.JavaCardRMIConnection;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

import com.sun.javacard.samples.RMIDemo.Purse;


public class JCRMIMIDlet extends MIDlet implements CommandListener, Runnable {
    private final String kRMIURL = "jcrmi:0;AID=a0.0.0.0.62.3.1.c.8.1";
    private JavaCardRMIConnection mConnection;
    private Display mDisplay;
    private Form mMainForm;
    private Command mExitCommand;
    private Command mGoCommand;
    private Command mBackCommand;
    private Form mProgressForm;

    public JCRMIMIDlet() {
        mExitCommand = new Command("Exit", Command.EXIT, 0);
        mGoCommand = new Command("Go", Command.SCREEN, 0);
        mBackCommand = new Command("Back", Command.BACK, 0);

        mMainForm = new Form("JCRMI Example");
        mMainForm.append("Press Go to use the SATSA-JCRMI API " +
            "to connect to a card application.");
        mMainForm.addCommand(mExitCommand);
        mMainForm.addCommand(mGoCommand);
        mMainForm.setCommandListener(this);
    }

    public void startApp() {
        mDisplay = Display.getDisplay(this);

        mDisplay.setCurrent(mMainForm);
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
    }

    public void commandAction(Command c, Displayable s) {
        if (c == mExitCommand) {
            notifyDestroyed();
        } else if (c == mGoCommand) {
            mProgressForm = new Form("Working...");
            mDisplay.setCurrent(mProgressForm);

            Thread t = new Thread(this);
            t.start();
        } else if (c == mBackCommand) {
            mDisplay.setCurrent(mMainForm);
        }
    }

    public void run() {
        try {
            setProgress("Opening a connection");
            mConnection = (JavaCardRMIConnection)Connector.open(kRMIURL);

            setProgress("Getting an initial reference");

            Purse purse = (Purse)mConnection.getInitialReference();

            short balance = purse.getBalance();
            setProgress("Balance = " + balance);

            setProgress("Crediting 20");
            purse.credit((short)20);
            setProgress("Debiting 15");
            purse.debit((short)15);

            balance = purse.getBalance();
            setProgress("Balance = " + balance);

            setProgress("Setting account number to 54321");
            purse.setAccountNumber(new byte[] { 5, 4, 3, 2, 1 });

            setProgress("Closing connection");
            mConnection.close();

            mProgressForm.setTitle("Working...done.");
            mProgressForm.addCommand(mBackCommand);
            mProgressForm.setCommandListener(this);
        } catch (Exception e) {
            try {
                mConnection.close();
            } catch (Throwable t) {
            }

            Form f = new Form("Exception");
            f.append(e.toString());
            f.addCommand(mBackCommand);
            f.setCommandListener(this);
            mDisplay.setCurrent(f);
        }
    }

    private void setProgress(String s) {
        StringItem si = new StringItem(null, s);
        si.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
        mProgressForm.append(si);
    }
}
